import EventEmitter from "eventemitter3";
import Constants from "../util/Constants.js";
import Utils from "../util/Utils.js";
import { Peer } from "./Peer.js";
import { TankPacket } from "../packets/TankPacket.js";
import { WebServer } from "./WebServer.js";
import { createRequire } from "node:module";
const Native = createRequire(import.meta.url)("../../build/Release/gtjs.node").Client;
class Client extends EventEmitter {
    _client;
    config;
    cache;
    constructor(options) {
        super();
        this.config = {
            plugins: options?.plugins ?? [],
            https: {
                ip: options?.https?.ip ?? "127.0.0.1",
                port: options?.https?.port ?? 17091,
                enable: options?.https?.enable ?? true,
                type2: options?.https?.type2 ?? false
            },
            enet: {
                ip: options?.enet?.ip ?? "127.0.0.1",
                port: options?.enet?.port ?? 17091,
                maxPeers: options?.enet?.maxPeers ?? 1024,
                useNewPacket: {
                    asClient: options?.enet?.useNewPacket?.asClient ?? false
                }
            }
        };
        this.config.plugins;
        this._client = new Native(this.config.enet?.ip, this.config.enet?.port);
        this.cache = {
            players: new Map()
        };
    }
    on(event, listener) {
        return super.on(event, listener);
    }
    send(id, count, packets) {
        return this._client.send(id, count, packets);
    }
    connect(ipAddress, port, peerID) {
        return this._client.connect(ipAddress, port, peerID);
    }
    toggleNewPacket() {
        return this._client.toggleNewPacket();
    }
    emitter(emit) {
        return this._client.setEmit(emit);
    }
    listen() {
        try {
            this._client.create(this.config?.enet?.maxPeers, this.config?.enet?.useNewPacket?.asClient);
            this.emitter(this.emit.bind(this));
            const acceptPromise = () => new Promise((resolve) => setImmediate(() => resolve(this._client.service())));
            const loop = async () => {
                while (true)
                    await acceptPromise();
            };
            loop();
            this._startWeb();
            this._handleEvent();
            this.emit("ready");
        }
        catch {
            this.emit("error", new Error("Failed to initialize ENet server"));
        }
    }
    _startWeb() {
        if (!this.config.https)
            return;
        if (this.config.https.enable)
            WebServer(this.config?.https?.ip, this.config?.https?.port, this.config.https.type2);
    }
    _handleEvent() {
        // Initialize plugins
        if (this.config?.plugins?.length)
            this.config.plugins.forEach((v) => v.init(this));
        this.on("connect", (netID) => {
            this.cache.players?.set(netID, netID);
            if (this.config.plugins?.length)
                this.config.plugins.forEach((v) => v.onConnect(netID));
        });
        this.on("disconnect", (netID) => {
            this.cache.players?.delete(netID);
            if (this.config.plugins?.length)
                this.config.plugins.forEach((v) => v.onDisconnect(netID));
        });
        this.on("raw", (netID, data) => {
            if (this.config.plugins?.length)
                this.config.plugins.forEach((v) => v.onRaw(netID, data));
            const type = data.readInt32LE();
            const peer = new Peer(this, netID);
            const PacketTypes = Constants.PacketTypes;
            switch (type) {
                case PacketTypes.STR: {
                    const parsed = Utils.parseText(data);
                    this.emit("text", peer, parsed);
                    break;
                }
                case PacketTypes.ACTION: {
                    const parsed = Utils.parseText(data);
                    this.emit("action", peer, parsed);
                    break;
                }
                case PacketTypes.TANK: {
                    if (data.length < 60) {
                        this.emit("error", new Error("Received invalid tank packet"), data);
                        return peer.disconnect();
                    }
                    const tank = TankPacket.fromBuffer(data);
                    this.emit("tank", peer, tank);
                    break;
                }
                default: {
                    this.emit("error", new Error(`Got unknown packet type of ${type}`), data);
                    break;
                }
            }
        });
    }
}
export { Client };
